const nodePath = require('path');
const _ = require('lodash');
const globby = require('globby');
const { getEncoding } = require('istextorbinary');

const Validator = require('./Validator');
const Configurator = require('./Configurator');
const Finder = require('./Finder');
const Patcher = require('./Patcher');
const Logger = require('./Logger');
const DateFormatter = require('./DateFormatter');
const helpers = require('./helpers');

const { fs } = helpers;

class ReplaceInFiles {
  async static getPathsToFiles({ files, optionsForFiles }) {
    const paths = await globby(files, optionsForFiles);
    return paths;
  }

  async static pipesHandlerActions({
    mainReplaceOptions,
    pathsToFiles,
    pipeReplaceOptions,
  }) {
    for (let i = 0, step = 1; i < pipeReplaceOptions.length; i++, step++) {
      const srcReplaceOptions = pipeReplaceOptions[i];
      const replaceOptions = _.assign(
        {},
        mainReplaceOptions,
        srcReplaceOptions,
        { step }
      );
      await ReplaceInFiles.handlerActions(replaceOptions, pathsToFiles);
    }
  }

  async static handlerActions(replaceOptions, pathsToFiles) {
    await _.map(pathsToFiles, path => ReplaceInFiles.handlerAction(
      replaceOptions,
      path
    ));
  }

  async static handlerAction(
    replaceOptions,
    path
  ) {
    const {
      shouldSkipBinaryFiles,
      encoding,
      from,
      onlyFindPathsWithoutReplace,
      replaceFileOnlyIfMatchRegxpInFile: regexp,
      to,
      saveOldFile,
      step,
    } = replaceOptions;
    const data = await fs.readFile(path, encoding);

    if (shouldSkipBinaryFiles && getEncoding(data) === 'binary') {
      return;
    }

    if (onlyFindPathsWithoutReplace) {
      ReplaceInFiles.findMatches({
        data,
        from,
        path,
        step,
      });
      return;
    }


    const isReplaceFile = _.some([
      regexp === null,
      regexp !== null && Finder.isFindRegxInString(data, regexp)
    ], Boolean);

    if (isReplaceFile) {
      await ReplaceInFiles.replaceMatches({
        data,
        encoding,
        from,
        path,
        saveOldFile,
        step,
        to,
      });
    }
  }

  static findMatches({
    data,
    from,
    path,
    step,
  }) {
    new Finder({
      data,
      from,
      path,
      step,
    }).run();
  }

  static getOldFilePath({ path, birthtime, ctime }) {
    const ext = nodePath.extname(path) || '';
    const startExt = -ext.length || path.length;
    const date = birthtime || ctime || new Date();
    const postfix = new DateFormatter({ date }).run();
    const oldFilePath = `${path.slice(0, startExt)}-${postfix}${ext}`;

    return oldFilePath;
  }

  async static saveOldFile({ path, data }) {
    const { birthtime, ctime } = await fs.stat(path);
    const oldFilePath = ReplaceInFiles.getOldFilePath({ path, birthtime, ctime });


    await fs.writeFile(oldFilePath, data);
  }

  async static replaceMatches({
    data,
    encoding,
    from,
    path,
    saveOldFile,
    step,
    to,
  }) {
    if (saveOldFile) {
      await ReplaceInFiles.saveOldFile({ path, data });
    }

    const updated = new Patcher({
      data,
      from,
      path,
      step,
      to,
    }).run();

    await fs.writeFile(path, updated, encoding);
  }

  constructor(options) {
    this.options = options;

    this.logger = null;
    this.loggerOptions = null;
    this.findFilesOptions = null;
    this.replaceOptions = null;
    this.pipeReplaceOptions = null;
  }

  validateOptions() {
    new Validator(this.options).run();
  }

  setConfings() {
    const {
      findFilesOptions,
      loggerOptions,
      replaceOptions,
    } = new Configurator(this.options).run();

    this.findFilesOptions = findFilesOptions;
    this.loggerOptions = loggerOptions;
    this.replaceOptions = replaceOptions;
  }

  * initSettings() {
    this.validateOptions();
    this.setConfings();

    this.logger = new Logger({
      loggerOptions: this.loggerOptions,
      replaceInFilesMainOptions: this.options
    });
    this.logger.run();

    await helpers.delay();
    const allOptions = this.logger.getReplaceInFilesOptions();
    this.pipeReplaceOptions = allOptions.slice(1);
    this.logger.setCountOfMatchesByPathsLength(allOptions.length);
  }

  * run() {
    await this.initSettings();

    const pathsToFiles = await ReplaceInFiles.getPathsToFiles(this.findFilesOptions);
    await ReplaceInFiles.handlerActions(this.replaceOptions, pathsToFiles);
    await ReplaceInFiles.pipesHandlerActions({
      mainReplaceOptions: this.replaceOptions,
      pathsToFiles,
      pipeReplaceOptions: this.pipeReplaceOptions,
    });

    return this.logger.getResult();
  }
}

module.exports = ReplaceInFiles;
